@@ -769,7 +769,7 @@ impl<'a> EgraphPass<'a> {
                             debug_assert!(self.func.layout.inst_block(i).is_some());
                         }
                         ValueDef::Union(..) => {
-                            panic!("egraph union node {} still reachable at {}!", arg, inst);
+                            panic!("egraph union node {arg} still reachable at {inst}!");
                         }
                         _ => {}
                     })
