@@ -15,14 +15,14 @@
 
 use crate::prelude::*;
 use crate::{
-    limits::*, BinaryReaderError, Encoding, FromReader, FunctionBody, HeapType, Parser, Payload,
-    RefType, Result, SectionLimited, ValType, WASM_COMPONENT_VERSION, WASM_MODULE_VERSION,
+    limits::*, AbstractHeapType, BinaryReaderError, Encoding, FromReader, FunctionBody, HeapType,
+    Parser, Payload, RefType, Result, SectionLimited, ValType, WasmFeatures,
+    WASM_COMPONENT_VERSION, WASM_MODULE_VERSION,
 };
 use ::core::mem;
 use ::core::ops::Range;
 use ::core::sync::atomic::{AtomicUsize, Ordering};
 use alloc::sync::Arc;
-use bitflags::bitflags;
 
 /// Test whether the given buffer contains a valid WebAssembly module or component,
 /// analogous to [`WebAssembly.validate`][js] in the JS API.
@@ -219,161 +219,6 @@ impl Default for State {
     }
 }
 
-macro_rules! define_wasm_features {
-    (
-        $(#[$outer:meta])*
-        pub struct WasmFeatures: $repr:ty {
-            $(
-                $(#[$inner:ident $($args:tt)*])*
-                pub $field:ident: $const:ident($flag:expr) = $default:expr;
-            )*
-        }
-    ) => {
-        bitflags! {
-            $(#[$outer])*
-            pub struct WasmFeatures: $repr {
-                $(
-                    $(#[$inner $($args)*])*
-                    #[doc = "\nDefaults to `"]
-                    #[doc = stringify!($default)]
-                    #[doc = "`.\n"]
-                    const $const = $flag;
-                )*
-            }
-        }
-
-        impl Default for WasmFeatures {
-            #[inline]
-            fn default() -> Self {
-                let mut features = WasmFeatures::empty();
-                $(
-                    features.set(WasmFeatures::$const, $default);
-                )*
-                features
-            }
-        }
-
-        impl WasmFeatures {
-            /// Construct a bit-packed `WasmFeatures` from the inflated struct version.
-            #[inline]
-            pub fn from_inflated(inflated: WasmFeaturesInflated) -> Self {
-                let mut features = WasmFeatures::empty();
-                $(
-                    features.set(WasmFeatures::$const, inflated.$field);
-                )*
-                features
-            }
-
-            /// Inflate these bit-packed features into a struct with a field per
-            /// feature.
-            ///
-            /// Although the inflated struct takes up much more memory than the
-            /// bit-packed version, its fields can be exhaustively matched
-            /// upon. This makes it useful for temporarily checking against,
-            /// while keeping the bit-packed version as the method of storing
-            /// the features for longer periods of time.
-            #[inline]
-            pub fn inflate(&self) -> WasmFeaturesInflated {
-                WasmFeaturesInflated {
-                    $(
-                        $field: self.contains(WasmFeatures::$const),
-                    )*
-                }
-            }
-        }
-
-        /// Inflated version of [`WasmFeatures`][crate::WasmFeatures] that
-        /// allows for exhaustive matching on fields.
-        pub struct WasmFeaturesInflated {
-            $(
-                $(#[$inner $($args)*])*
-                #[doc = "\nDefaults to `"]
-                #[doc = stringify!($default)]
-                #[doc = "`.\n"]
-                pub $field: bool,
-            )*
-        }
-    };
-}
-
-define_wasm_features! {
-    /// Flags for features that are enabled for validation.
-    #[derive(Hash, Debug, Copy, Clone)]
-    pub struct WasmFeatures: u32 {
-        /// The WebAssembly `mutable-global` proposal.
-        pub mutable_global: MUTABLE_GLOBAL(1) = true;
-        /// The WebAssembly `saturating-float-to-int` proposal.
-        pub saturating_float_to_int: SATURATING_FLOAT_TO_INT(1 << 1) = true;
-        /// The WebAssembly `sign-extension-ops` proposal.
-        pub sign_extension: SIGN_EXTENSION(1 << 2) = true;
-        /// The WebAssembly reference types proposal.
-        pub reference_types: REFERENCE_TYPES(1 << 3) = true;
-        /// The WebAssembly multi-value proposal.
-        pub multi_value: MULTI_VALUE(1 << 4) = true;
-        /// The WebAssembly bulk memory operations proposal.
-        pub bulk_memory: BULK_MEMORY(1 << 5) = true;
-        /// The WebAssembly SIMD proposal.
-        pub simd: SIMD(1 << 6) = true;
-        /// The WebAssembly Relaxed SIMD proposal.
-        pub relaxed_simd: RELAXED_SIMD(1 << 7) = true;
-        /// The WebAssembly threads proposal.
-        pub threads: THREADS(1 << 8) = true;
-        /// The WebAssembly shared-everything-threads proposal; includes new
-        /// component model built-ins.
-        pub shared_everything_threads: SHARED_EVERYTHING_THREADS(1 << 9) = false;
-        /// The WebAssembly tail-call proposal.
-        pub tail_call: TAIL_CALL(1 << 10) = true;
-        /// Whether or not floating-point instructions are enabled.
-        ///
-        /// This is enabled by default can be used to disallow floating-point
-        /// operators and types.
-        ///
-        /// This does not correspond to a WebAssembly proposal but is instead
-        /// intended for embeddings which have stricter-than-usual requirements
-        /// about execution. Floats in WebAssembly can have different NaN patterns
-        /// across hosts which can lead to host-dependent execution which some
-        /// runtimes may not desire.
-        pub floats: FLOATS(1 << 11) = true;
-        /// The WebAssembly multi memory proposal.
-        pub multi_memory: MULTI_MEMORY(1 << 12) = true;
-        /// The WebAssembly exception handling proposal.
-        pub exceptions: EXCEPTIONS(1 << 13) = false;
-        /// The WebAssembly memory64 proposal.
-        pub memory64: MEMORY64(1 << 14) = false;
-        /// The WebAssembly extended_const proposal.
-        pub extended_const: EXTENDED_CONST(1 << 15) = false;
-        /// The WebAssembly component model proposal.
-        pub component_model: COMPONENT_MODEL(1 << 16) = true;
-        /// The WebAssembly typed function references proposal.
-        pub function_references: FUNCTION_REFERENCES(1 << 17) = false;
-        /// The WebAssembly memory control proposal.
-        pub memory_control: MEMORY_CONTROL(1 << 18) = false;
-        /// The WebAssembly gc proposal.
-        pub gc: GC(1 << 19) = false;
-        /// The WebAssembly [custom-page-sizes
-        /// proposal](https://github.com/WebAssembly/custom-page-sizes).
-        pub custom_page_sizes: CUSTOM_PAGE_SIZES(1 << 20) = false;
-        /// Support for the `value` type in the component model proposal.
-        pub component_model_values: COMPONENT_MODEL_VALUES(1 << 21) = false;
-        /// Support for the nested namespaces and projects in component model names.
-        pub component_model_nested_names: COMPONENT_MODEL_NESTED_NAMES(1 << 22) = false;
-    }
-}
-
-impl From<WasmFeaturesInflated> for WasmFeatures {
-    #[inline]
-    fn from(inflated: WasmFeaturesInflated) -> Self {
-        Self::from_inflated(inflated)
-    }
-}
-
-impl From<WasmFeatures> for WasmFeaturesInflated {
-    #[inline]
-    fn from(features: WasmFeatures) -> Self {
-        features.inflate()
-    }
-}
-
 impl WasmFeatures {
     /// NOTE: This only checks that the value type corresponds to the feature set!!
     ///
@@ -383,7 +228,7 @@ impl WasmFeatures {
         match ty {
             ValType::I32 | ValType::I64 => Ok(()),
             ValType::F32 | ValType::F64 => {
-                if self.contains(Self::FLOATS) {
+                if self.floats() {
                     Ok(())
                 } else {
                     Err("floating-point support is disabled")
@@ -391,7 +236,7 @@ impl WasmFeatures {
             }
             ValType::Ref(r) => self.check_ref_type(r),
             ValType::V128 => {
-                if self.contains(Self::SIMD) {
+                if self.simd() {
                     Ok(())
                 } else {
                     Err("SIMD support is not enabled")
@@ -401,58 +246,70 @@ impl WasmFeatures {
     }
 
     pub(crate) fn check_ref_type(&self, r: RefType) -> Result<(), &'static str> {
-        if !self.contains(Self::REFERENCE_TYPES) {
+        if !self.reference_types() {
             return Err("reference types support is not enabled");
         }
-        match (r.heap_type(), r.is_nullable()) {
-            // funcref/externref only require `reference-types`.
-            (HeapType::Func, true) | (HeapType::Extern, true) => Ok(()),
+        match r.heap_type() {
+            HeapType::Concrete(_) => {
+                // Note that `self.gc_types()` is not checked here because
+                // concrete pointers to function types are allowed. GC types
+                // are disallowed by instead rejecting the definition of
+                // array/struct types and only allowing the definition of
+                // function types.
 
-            // Non-nullable func/extern references requires the
-            // `function-references` proposal.
-            (HeapType::Func | HeapType::Extern, false) => {
-                if self.contains(Self::FUNCTION_REFERENCES) {
-                    Ok(())
-                } else {
-                    Err("function references required for non-nullable types")
-                }
-            }
-
-            // Indexed types require either the function-references or gc
-            // proposal as gc implies function references here.
-            (HeapType::Concrete(_), _) => {
-                if self.contains(Self::FUNCTION_REFERENCES) || self.contains(Self::GC) {
+                // Indexed types require either the function-references or gc
+                // proposal as gc implies function references here.
+                if self.function_references() || self.gc() {
                     Ok(())
                 } else {
                     Err("function references required for index reference types")
                 }
             }
-
-            // These types were added in the gc proposal.
-            (
-                HeapType::Any
-                | HeapType::None
-                | HeapType::Eq
-                | HeapType::Struct
-                | HeapType::Array
-                | HeapType::I31
-                | HeapType::NoExtern
-                | HeapType::NoFunc,
-                _,
-            ) => {
-                if self.contains(Self::GC) {
-                    Ok(())
-                } else {
-                    Err("heap types not supported without the gc feature")
+            HeapType::Abstract { shared, ty } => {
+                use AbstractHeapType::*;
+                if shared && !self.shared_everything_threads() {
+                    return Err(
+                        "shared reference types require the shared-everything-threads proposal",
+                    );
                 }
-            }
 
-            // These types were added in the exception-handling proposal.
-            (HeapType::Exn | HeapType::NoExn, _) => {
-                if self.contains(Self::EXCEPTIONS) {
-                    Ok(())
-                } else {
-                    Err("exception refs not supported without the exception handling feature")
+                // Apply the "gc-types" feature which disallows all heap types
+                // except exnref/funcref.
+                if !self.gc_types() && ty != Func && ty != Exn {
+                    return Err("gc types are disallowed but found type which requires gc");
+                }
+
+                match (ty, r.is_nullable()) {
+                    // funcref/externref only require `reference-types`.
+                    (Func, true) | (Extern, true) => Ok(()),
+
+                    // Non-nullable func/extern references requires the
+                    // `function-references` proposal.
+                    (Func | Extern, false) => {
+                        if self.function_references() {
+                            Ok(())
+                        } else {
+                            Err("function references required for non-nullable types")
+                        }
+                    }
+
+                    // These types were added in the gc proposal.
+                    (Any | None | Eq | Struct | Array | I31 | NoExtern | NoFunc, _) => {
+                        if self.gc() {
+                            Ok(())
+                        } else {
+                            Err("heap types not supported without the gc feature")
+                        }
+                    }
+
+                    // These types were added in the exception-handling proposal.
+                    (Exn | NoExn, _) => {
+                        if self.exceptions() {
+                            Ok(())
+                        } else {
+                            Err("exception refs not supported without the exception handling feature")
+                        }
+                    }
                 }
             }
         }
@@ -610,7 +467,11 @@ impl Validator {
     pub fn validate_all(&mut self, bytes: &[u8]) -> Result<Types> {
         let mut functions_to_validate = Vec::new();
         let mut last_types = None;
-        for payload in Parser::new(0).parse_all(bytes) {
+        let mut parser = Parser::new(0);
+        let _ = &mut parser;
+        #[cfg(feature = "features")]
+        parser.set_features(self.features);
+        for payload in parser.parse_all(bytes) {
             match self.payload(&payload?)? {
                 ValidPayload::Func(a, b) => {
                     functions_to_validate.push((a, b));
@@ -773,7 +634,7 @@ impl Validator {
                 }
             }
             Encoding::Component => {
-                if !self.features.contains(WasmFeatures::COMPONENT_MODEL) {
+                if !self.features.component_model() {
                     bail!(
                         range.start,
                         "unknown binary version and encoding combination: {num:#x} and 0x1, \
@@ -831,7 +692,17 @@ impl Validator {
             Order::Import,
             section,
             "import",
-            |_, _, _, _, _| Ok(()), // add_import will check limits
+            |state, _, _, count, offset| {
+                check_max(
+                    state.module.imports.len(),
+                    count,
+                    MAX_WASM_IMPORTS,
+                    "imports",
+                    offset,
+                )?;
+                state.module.assert_mut().imports.reserve(count as usize);
+                Ok(())
+            },
             |state, features, types, import, offset| {
                 state
                     .module
@@ -919,7 +790,7 @@ impl Validator {
     ///
     /// This method should only be called when parsing a module.
     pub fn tag_section(&mut self, section: &crate::TagSectionReader<'_>) -> Result<()> {
-        if !self.features.contains(WasmFeatures::EXCEPTIONS) {
+        if !self.features.exceptions() {
             return Err(BinaryReaderError::new(
                 "exceptions proposal not enabled",
                 section.range().start,
@@ -1589,7 +1460,7 @@ impl Validator {
     {
         let offset = section.range().start;
 
-        if !self.features.contains(WasmFeatures::COMPONENT_MODEL) {
+        if !self.features.component_model() {
             return Err(BinaryReaderError::new(
                 "component model feature is not enabled",
                 offset,
@@ -1690,6 +1561,8 @@ mod tests {
                 initial: 10,
                 maximum: None,
                 element_type: RefType::FUNCREF,
+                table64: false,
+                shared: false,
             }
         );
 
