@@ -204,6 +204,7 @@ struct WasmFeatures {
     custom_page_sizes: bool,
     component_model_more_flags: bool,
     component_model_multiple_returns: bool,
+    gc_types: bool,
 }
 
 impl Metadata<'_> {
@@ -232,13 +233,14 @@ impl Metadata<'_> {
             component_model_more_flags,
             component_model_multiple_returns,
             legacy_exceptions,
+            gc_types,
 
             // Always on; we don't currently have knobs for these.
             mutable_global: _,
             saturating_float_to_int: _,
             sign_extension: _,
             floats: _,
-        } = engine.config().features.inflate();
+        } = engine.features().inflate();
 
         // These features are not implemented in Wasmtime yet. We match on them
         // above so that once we do implement support for them, we won't
@@ -272,6 +274,7 @@ impl Metadata<'_> {
                 custom_page_sizes,
                 component_model_more_flags,
                 component_model_multiple_returns,
+                gc_types,
             },
         }
     }
@@ -281,7 +284,7 @@ impl Metadata<'_> {
         self.check_shared_flags(engine)?;
         self.check_isa_flags(engine)?;
         self.check_tunables(&engine.tunables())?;
-        self.check_features(&engine.config().features)?;
+        self.check_features(&engine.features())?;
         Ok(())
     }
 
@@ -477,31 +480,25 @@ impl Metadata<'_> {
             custom_page_sizes,
             component_model_more_flags,
             component_model_multiple_returns,
+            gc_types,
         } = self.features;
 
         use wasmparser::WasmFeatures as F;
-        Self::check_cfg_bool(
-            cfg!(feature = "gc"),
-            "gc",
+        Self::check_bool(
             reference_types,
             other.contains(F::REFERENCE_TYPES),
             "WebAssembly reference types support",
         )?;
-        Self::check_cfg_bool(
-            cfg!(feature = "gc"),
-            "gc",
+        Self::check_bool(
             function_references,
             other.contains(F::FUNCTION_REFERENCES),
             "WebAssembly function-references support",
         )?;
-        Self::check_cfg_bool(
-            cfg!(feature = "gc"),
-            "gc",
+        Self::check_bool(
             gc,
             other.contains(F::GC),
             "WebAssembly garbage collection support",
         )?;
-
         Self::check_bool(
             multi_value,
             other.contains(F::MULTI_VALUE),
@@ -568,6 +565,13 @@ impl Metadata<'_> {
             other.contains(F::COMPONENT_MODEL_MULTIPLE_RETURNS),
             "WebAssembly component model support for multiple returns",
         )?;
+        Self::check_cfg_bool(
+            cfg!(feature = "gc"),
+            "gc",
+            gc_types,
+            other.contains(F::GC_TYPES),
+            "support for WebAssembly gc types",
+        )?;
 
         Ok(())
     }
@@ -632,7 +636,7 @@ mod test {
 
         match metadata.check_compatible(&engine) {
             Ok(_) => unreachable!(),
-            Err(e) => assert!(format!("{:?}", e).starts_with(
+            Err(e) => assert!(format!("{e:?}").starts_with(
                 "\
 compilation settings of module incompatible with native host
 
