@@ -325,7 +325,7 @@ impl<T> Linker<T> {
             match item_def {
                 TypeDef::ComponentFunc(_) => {
                     let fully_qualified_name = parent_instance
-                        .map(|parent| format!("{}#{}", parent, item_name))
+                        .map(|parent| format!("{parent}#{item_name}"))
                         .unwrap_or_else(|| item_name.to_owned());
                     linker.func_new(&item_name, move |_, _, _| {
                         bail!("unknown import: `{fully_qualified_name}` has not been defined")
@@ -621,6 +621,37 @@ impl<T> LinkerInstance<'_, T> {
         Ok(())
     }
 
+    /// Identical to [`Self::resource`], except that it takes an async destructor.
+    #[cfg(feature = "async")]
+    pub fn resource_async<F>(&mut self, name: &str, ty: ResourceType, dtor: F) -> Result<()>
+    where
+        F: for<'a> Fn(
+                StoreContextMut<'a, T>,
+                u32,
+            ) -> Box<dyn Future<Output = Result<()>> + Send + 'a>
+            + Send
+            + Sync
+            + 'static,
+    {
+        assert!(
+            self.engine.config().async_support,
+            "cannot use `resource_async` without enabling async support in the config"
+        );
+        let dtor = Arc::new(crate::func::HostFunc::wrap_inner(
+            &self.engine,
+            move |mut cx: crate::Caller<'_, T>, (param,): (u32,)| {
+                let async_cx = cx.as_context_mut().0.async_cx().expect("async cx");
+                let mut future = Pin::from(dtor(cx.as_context_mut(), param));
+                match unsafe { async_cx.block_on(future.as_mut()) } {
+                    Ok(Ok(())) => Ok(()),
+                    Ok(Err(trap)) | Err(trap) => Err(trap),
+                }
+            },
+        ));
+        self.insert(name, Definition::Resource(ty, dtor))?;
+        Ok(())
+    }
+
     /// Defines a nested instance within this instance.
     ///
     /// This can be used to describe arbitrarily nested levels of instances
