@@ -9,12 +9,14 @@
 #![allow(missing_docs, unreachable_code)]
 
 mod anyref;
+mod arrayref;
 mod externref;
 mod i31;
 mod rooting;
 mod structref;
 
 pub use anyref::*;
+pub use arrayref::*;
 pub use externref::*;
 pub use i31::*;
 pub use rooting::*;
