@@ -2,12 +2,14 @@
 //! feature is enabled.
 
 mod anyref;
+mod arrayref;
 mod externref;
 mod i31;
 mod rooting;
 mod structref;
 
 pub use anyref::*;
+pub use arrayref::*;
 pub use externref::*;
 pub use i31::*;
 pub use rooting::*;
