@@ -1019,7 +1019,7 @@ impl<T> Linker<T> {
             Entry::Occupied(_) if !self.allow_shadowing => {
                 let module = &self.strings[key.module];
                 let desc = match self.strings.get(key.name) {
-                    Some(name) => format!("{}::{}", module, name),
+                    Some(name) => format!("{module}::{name}"),
                     None => module.to_string(),
                 };
                 bail!("import of `{}` defined twice", desc)
@@ -1226,7 +1226,7 @@ impl<T> Linker<T> {
     pub fn iter<'a: 'p, 'p>(
         &'a self,
         mut store: impl AsContextMut<Data = T> + 'p,
-    ) -> impl Iterator<Item = (&str, &str, Extern)> + 'p {
+    ) -> impl Iterator<Item = (&'a str, &'a str, Extern)> + 'p {
         self.map.iter().map(move |(key, item)| {
             let store = store.as_context_mut();
             (
